************************************************************************
*     Hough Functions for given Equivalent Depth
*     of Zonal Wave Number 'M'.
*     (Eigen function problem for frequency)
*     Plot horizontal wind field and geopotential field(=Hough Function)
*       original   Ishioka,K.
*       last revised   Mar 1993   Horinouchi,T. 
*       (addapted to dcl-5, Apr 1997, Horiouchi)
***********************************************************************
c     parameters
c       MM: maximum total wave number for calculation 
c       JH: meridional resolution(grid number for hemisphere) to plot
c    +      geopotential field (should be .le.MM)
c       M : zonal wave number
c       NS: symmetric(=0) or antisymmetric(=1) 
c       HH: equivalent depth (=1.4D0*H)
c     variables
c       E: eigen values which correspond nondimensional frequency 
c       EV: eigen vectors 
c       A,B,C: coefficients for velocity potential, stream function,
c    +     geopotential.
c       U,V: horizontal wind , Z: geopotential
c       P(*,1,N): Legendre associated func. zonal w.n. M total wn N
c       P(*,2,N): (1-x**2)(d/dx)P(*,1,N)
*====================================================================== 
      IMPLICIT REAL*8(A-H,O-Q,S-Z),REAL(R)
      PARAMETER(NP=6,JH=50,IM=72,JG=9,IG=36,MM=60,LM=3*(MM/2+1))
      DIMENSION D(LM*(LM+1)/2),E(LM),EV(LM,LM),VW(2*LM),EVD(LM)
      DIMENSION A(MM),B(MM),C(MM)
      DIMENSION X(-JH:JH,2),P(-JH:JH,2,0:MM),T(0:IM,2)
      DIMENSION Y(-JG:JG,2),Q(-JG:JG,2,0:MM),S(0:IG,2)
      DIMENSION U(-JG:JG),V(-JG:JG),Z(-JH:JH)
      DIMENSION RU(0:IG,-JG:JG),RV(0:IG,-JG:JG),RZ(0:IM,-JH:JH)
      DIMENSION LP(NP)
 
      PI=4*ATAN(1D0)
      OME=2*PI/86400
      AA=6.37D6
      G=9.7D0
      PHASE=3*PI/4
 
      WRITE(6,*) 'zonal wave number(integer);'
      READ(5,*) M
      WRITE(6,*) 'symmetric(=0) or antisymmetric(=1)'
      READ(5,*) IS
      WRITE(6,*) 'Eqivalent Depth (m)'
      READ(5,*) HH
      EPS=4*AA*AA*OME*OME/(G*HH)
 
      CALL PREP(MM,M,JH,IM,X,P,T,PHASE)
      CALL PREP(MM,M,JG,IG,Y,Q,S,PHASE)
      CALL PARAM(MM,M,IS,KM,NS,ID)
      CALL MATRIX(D,M,KM,NS,ID,EPS)
      CALL DSEIG1(D,KM,E,EV,LM,KMD,VW,ICON)
      CALL BOBBLE(LM,KM,E,EV,EVD)
 
      CALL GROPEN
      CALL TITLE(M,IS,HH)
**      CALL SLDIV('T',1,2)
      CALL SLRAT(1.0,0.6)
 
* --- set Rossby waves
      INDEX=1
      IDUM=0
      L=1
   30 CONTINUE
      ED=E(L)
      IF(ED.GE.-M/SQRT(EPS)) THEN
        IF(ED.LT.-1D-6) THEN
          LP(INDEX)=L
          INDEX=INDEX+1
        ELSE
          IDUM=IDUM+1
        END IF
      END IF
      L=L+1
      IF(INDEX+IDUM.LE.2) GOTO 30
* --- set westward gravity waves
      DO 40 I=1,2
        LD=L-2-I
        IF(LD.GE.1 .AND. E(LD).LT.-1D-6) THEN
          LP(INDEX)=LD
          INDEX=INDEX+1
        ELSE
          IDUM=IDUM+1
        ENDIF
   40 CONTINUE
* --- set eastward gravity waves
   50 CONTINUE
      ED=E(L)
      IF(ED.GT.1D-6) THEN
        LP(INDEX)=L
        INDEX=INDEX+1
      END IF
      L=L+1
      IF(INDEX+IDUM.LE.NP) GOTO 50 

      DO 20 IP=1,INDEX-1
      L=LP(IP)
      CALL SETABC(MM,LM,KM,NS,ID,EPS,L,EV,A,B,C)
      CALL SETUVZ(MM,M,NS,JH,JG,Y,P,Q,A,B,C,U,V,Z)
      CALL GRID(JH,JG,IM,IG,T,S,U,V,Z,RU,RV,RZ)
      CALL GRAPH(JH,JG,IM,IG,RU,RV,RZ)
      CALL INFORM(KM,L,E,HH)
   20 CONTINUE
 
      CALL GRCLS
 
      STOP
      END
************************************************************************
      SUBROUTINE TITLE(M,IS,HH)
 
      IMPLICIT REAL*8(A-H,O-T,V-Z),REAL(U)
      CHARACTER CS*4,CH*15,CT*24
 
      CS(1:3)=' s='
      WRITE(CS(4:4),'(I1)') M
      CH=' h=****.***(km)'
      WRITE(CH(4:11),'(F8.3)') HH*1D-3
 
      CT='  (symmetric)'
      IF(NS.EQ.1) CT=' (antisymmetric)'
 
      CALL SLSTTL(CS//CH//CT,'T',0.0,-0.7,0.025,1)
      CALL SLSTTL('#PAGE','B',0.0,0.0,0.02,2)
 
      RETURN
      END
************************************************************************
      SUBROUTINE PARAM(MM,M,IS,KM,NS,ID)
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      IE=MOD(MM-M,2)
      KM=3*((MM-M)/2+1)-(1-IE)*(1+IS)
      ID=IS
      NS=M
 
      IF(M.EQ.0) THEN
      KM=KM-(2-IS)
      ID=1-IS
      NS=M+1
      END IF
 
      RETURN
      END
************************************************************************
      SUBROUTINE MATRIX(D,M,KM,NS,ID,EPS)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION D(KM*(KM+1)/2)
 
      R(N)=-1D0*M/(N*(N+1))
      S(N)=-SQRT(N*(N+1)/EPS)
      T(N)=SQRT(1D0*(N*N-1)*(N*N-M*M)/(4*N*N-1))/N
 
      DO 10 I=1,KM*(KM+1)/2
      D(I)=0
   10 CONTINUE
 
      N=NS
      L=1
      DO 20 K=1,KM
      JR=MOD(K-1-ID,3)
      IF(JR.NE.0) THEN
      D(L)=R(N)
      N=N+1
      END IF
      L=L+K+1
   20 CONTINUE
 
      N=NS+ID
      L=2
      DO 30 K=2,KM
      JS=MOD(K-2-ID,3)
      JT=MOD(K-ID,3)
      IF(JS.EQ.0) THEN
      D(L)=S(N)
      N=N+1
      ELSE IF(JT.EQ.0) THEN
      D(L)=T(N)
      N=N+1
      END IF
      L=L+K+1
   30 CONTINUE
 
      N=NS+2-ID
      L=4
      DO 40 K=3,KM
      JS=MOD(K-2-ID,3)
      IF(JS.EQ.0) THEN
      D(L)=T(N)
      N=N+2
      END IF
      L=L+K+1
   40 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GRID(JH,JG,IM,IG,T,S,U,V,Z,RU,RV,RZ)
 
      IMPLICIT REAL*8(A-H,O-Q,S-Z),REAL(R)
      DIMENSION T(0:IM,2),S(0:IG,2)
      DIMENSION U(-JG:JG),V(-JG:JG),Z(-JH:JH)
      DIMENSION RU(0:IG,-JG:JG),RV(0:IG,-JG:JG),RZ(0:IM,-JH:JH)
 
      DO 10 J=-JH,JH
      DO 10 I=0,IM
      RZ(I,J)=Z(J)*T(I,1)
   10 CONTINUE
 
      DO 20 J=-JG,JG
      DO 20 I=0,IG
      RU(I,J)=U(J)*S(I,1)
      RV(I,J)=V(J)*S(I,2)
   20 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETUVZ(MM,M,NS,JH,JG,Y,P,Q,A,B,C,U,V,Z)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(MM),B(MM),C(MM)
      DIMENSION P(-JH:JH,2,0:MM)
      DIMENSION Y(-JG:JG,2),Q(-JG:JG,2,0:MM)
      DIMENSION U(-JG:JG),V(-JG:JG),Z(-JH:JH)
 
      DO 10 J=-JH,JH
      Z(J)=0
   10 CONTINUE
 
      DO 20 J=-JG,JG
      U(J)=0
      V(J)=0
   20 CONTINUE
 
      DO 30 N=NS,MM
      DO 30 J=-JH,JH
      Z(J)=Z(J)+C(N)*P(J,1,N)
   30 CONTINUE
 
      DO 40 N=NS,MM
      DO 40 J=-JG+1,JG-1
      U(J)=U(J)+M*A(N)*Q(J,1,N)+B(N)*Q(J,2,N)
      V(J)=V(J)+M*B(N)*Q(J,1,N)+A(N)*Q(J,2,N)
   40 CONTINUE
 
      DO 50 J=-JG+1,JG-1
      U(J)=-U(J)/Y(J,2)
      V(J)=V(J)/Y(J,2)
   50 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE SETABC(MM,LM,KM,NS,ID,EPS,L,EV,A,B,C)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION EV(LM,KM),A(MM),B(MM),C(MM)
 
      F(N)=1/SQRT(EPS*N*(N+1))
 
      DO 10 N=NS,MM
      A(N)=0
      B(N)=0
      C(N)=0
   10 CONTINUE
 
      N=NS
      DO 20 K=1,KM
      J=MOD(K-ID,3)
      IF(J.EQ.1) THEN
      C(N)=EV(K,L)
      A(N)=F(N)*EV(K+1,L)
      N=N+1
      ELSE IF(J.EQ.0) THEN
      B(N)=F(N)*EV(K,L)
      N=N+1
      END IF
   20 CONTINUE
 
      RETURN
      END
************************************************************************
      SUBROUTINE GROPEN
 
      COMMON /UNIT/RX,RY
 
      CALL SGPWSN
      READ(5,*) IWS
**      IWS=-IWS
      CALL GROPN(IWS)
      CALL SLMGN(0.0,0.0,0.05,0.05)
      CALL SLDIV('T',2,2)
      CALL GLPSET('MSGLEV',2)
      CALL SGPSET('LFULL',.TRUE.)
      CALL SGPSET('LCNTL',.TRUE.)
      CALL SGPSET('LCORNER',.FALSE.)
      CALL UZFACT(0.7)
      CALL UZPSET('UXUSER',0.0)
      CALL UZPSET('UYUSER',0.0)
      CALL UDPSET('LMSG',.FALSE.)
      CALL UGPSET('INDEX',1)
      CALL UGPSET('LMSG',.FALSE.)
      CALL UGPSET('LUNIT',.TRUE.)
      CALL UGPGET('UXUNIT',RX)
      CALL UGPGET('UYUNIT',RY)
      CALL UGPSET('VXULOC',0.15)
      CALL UGPSET('VYULOC',0.02)
 
      RETURN
      END
********************************************************************** *
      SUBROUTINE BOBBLE(LM,KM,E,EV,EVD)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(KM),EV(LM,KM),EVD(KM)
 
   30 CONTINUE
      INDEX=0
      DO 20 K=1,KM-1
      IF(E(K).GT.E(K+1)) THEN
      ED=E(K)
      E(K)=E(K+1)
      E(K+1)=ED
      DO 10 L=1,KM
      EVD(L)=EV(L,K)
      EV(L,K)=EV(L,K+1)
      EV(L,K+1)=EVD(L)
   10 CONTINUE
      INDEX=1
      END IF
   20 CONTINUE
      IF(INDEX.EQ.1) GOTO 30

      RETURN
      END
************************************************************************
      SUBROUTINE GRAPH(JH,JG,IM,IG,RU,RV,RZ)
 
      IMPLICIT REAL*8(A-H,O-Q,S-Z),REAL(R)
      DIMENSION RU(0:IG,-JG:JG),RV(0:IG,-JG:JG),RZ(0:IM,-JH:JH)
 
      COMMON /UNIT/RX,RY
 
      CALL GRFRM
      CALL GRSVPT(0.05,0.95,0.1,0.55)
      CALL GRSWND(0.0,360.0,-90.0,90.0)
      CALL GRSTRN (1)
      CALL GRSTRF
      CALL UXAXDV('B',30.0,90.0)
      CALL UXAXDV('T',30.0,90.0)
      CALL UYAXDV('R',10.0,30.0)
      CALL UYAXDV('L',10.0,30.0)
      CALL UDCNTR(RZ,IM+1,IM+1,2*JH+1)
      CALL UGPSET('UXUNIT',RX)
      CALL UGPSET('UYUNIT',RY)
      CALL UGPSET('VXUNIT',0.04)
      CALL UGPSET('VYUNIT',0.04)
      CALL UGVECT(RU,IG+1,RV,IG+1,IG+1,2*JG+1)
 
      RETURN
      END
************************************************************************
      SUBROUTINE INFORM(KM,L,E,HH)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION E(KM)
      CHARACTER CS*15,CSGI*1,CH*15
 
      CS(1:2)=CSGI(169)//'='
      CS(12:15)='('//CSGI(5)//CSGI(151)//')'
      WRITE(CS(3:11),'(F9.5)') 2*E(L)
      CH=' h=****.***(km)'
      WRITE(CH(4:11),'(F8.3)') HH*1D-3
 
      CALL UXMTTL('T',CS,0.0)
**      CALL UXMTTL('T',CS//'  '//CH,0.0)
 
      RETURN
      END
************************************************************************
      SUBROUTINE PREP(MM,M,JH,IM,X,P,T,PHASE)
 
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(-JH:JH,2),P(-JH:JH,2,0:MM),T(0:IM,2)
 
      PI=4*ATAN(1D0)
 
      DO 10 J=-JH,JH
      X(J,1)=SIN(PI/2*J/JH)
      X(J,2)=COS(PI/2*J/JH)
      P(J,1,M)=1
   10 CONTINUE
      XD=1D-10
      X(JH,2)=XD
      X(JH,1)=SQRT(1-XD*XD)
      X(-JH,2)=XD
      X(-JH,1)=-X(JH,1)
 
      DO 20 MD=1,M
      DO 20 J=-JH,JH
      P(J,1,M)=SQRT(1D0*(2*MD+1)/(2*MD))*X(J,2)*P(J,1,M)
   20 CONTINUE
      DO 30 J=-JH,JH
      P(J,2,M)=-M*X(J,1)*P(J,1,M)
   30 CONTINUE
 
      DO 40 N=M+1,MM
      EA=SQRT(1D0*(2*N+1)/(2*N-1))
      EB=SQRT(1D0*(N+M)*(N-M))
      E1=EA/EB
      E2=EA*EB
      DO 40 J=-JH,JH
      P(J,1,N)=E1*(N*X(J,1)*P(J,1,N-1)-P(J,2,N-1))
      P(J,2,N)=E2*P(J,1,N-1)-N*X(J,1)*P(J,1,N)
   40 CONTINUE
 
      IF(M.NE.0) THEN
      DO 50 I=0,IM
      T(I,1)=SIN(2*PI*M*I/IM)
      T(I,2)=COS(2*PI*M*I/IM)
   50 CONTINUE
      ELSE
      DO 60 I=0,IM
      T(I,1)=-SIN(PHASE)
      T(I,2)=COS(PHASE)
   60 CONTINUE
      END IF
 
      RETURN
      END
