!  Data module for 2-D shallow water model on a staggered grid
!
!     h - u - h - u - h -      d: $B?e?<(B
!     |       |       |        g: $B=ENO2CB.EY(B
!     v   q   v   q   v        f: $B%3%j%*%j%Q%i%a%?(B
!     |       |       | 
!     h - u - h - u - h -
!     |       |       |
!
module sw2_data
    integer, parameter :: imx = 50, jmx = 50, im1 = imx+1, jm1 = jmx+1
    integer :: n

    real :: h(im1,jm1),  xh(im1),  yh(jm1)
    real :: u(imx,jm1),  xu(imx),  yu(im1)
    real :: v(im1,jmx),  xv(im1),  yv(jmx)

    real :: xmax = 10., ymax = 10.,  dx,  dy
    real :: p_time=0., dt, f, d=1., g=1., r0, b


  contains
!------------------- setup data ----------------------
    subroutine setup

      dx = xmax/imx 
      dy = ymax/jmx

      xu = (/ (dx*i+dx/2., i=0, imx-1) /)
      yu = (/ (dy*j,       j=0, jmx  ) /)
      xv = (/ (dx*i,       i=0, imx  ) /)
      yv = (/ (dy*j+dy/2., j=0, jmx-1) /)
      xh = (/ (dx*i,       i=0, imx  ) /)
      yh = (/ (dy*j,       j=0, jmx  ) /)
      
!!!      write(*,*)'b'
!!!     read (*,*) b

!!!      write(*,*)'n'
!!!     read (*,*) n

!!!      write(*,*)'f'
!!!      read (*,*) f
      
!!!      write(*,*)'omega'
!!!      read(*,*)omega

      open (1,file='kelvin1.data')
      read (1,*) b, n, f, omega
      close(1)


      if((f==0.).and.(b==0.))then
      r0 =999
      dt=min(dx/sqrt(g*d),dy/sqrt(g*d))

      else if((f==0.).and.(b/=0.))then
      r0 =999
      dt=min(dx/sqrt(g*d),dy/sqrt(g*d))

      else if((f/=0.).and.(b==0.))then
      r0 =999
      dt=min(dx/sqrt(g*d),dy/sqrt(g*d))

      else
      r0 =abs(sqrt(g*d)/f)
      dt = min(dx/sqrt(g*d), dy/sqrt(g*d),abs(1./f))

      end if

      dt=dt/2.

    end subroutine
!------------------ average_x ------------------------
    function a_x (var)
      real, dimension(:,:) :: var
      real, dimension(size(var,1)-1, size(var,2)) :: a_x
      
      im  = size(var,1)
      a_x = (var(1:im-1,:) + var(2:im,:))/2.
    end function a_x

!------------------ average_y ------------------------
    function a_y (var)
      real, dimension(:,:) :: var
      real, dimension(size(var,1), size(var,2)-1) :: a_y

      jm  = size(var,2)
      a_y = (var(:,1:jm-1) + var(:,2:jm))/2.
    end function a_y

!--------------- derivative_x ------------------
    function d_x (var)
      real, dimension(:,:) :: var
      real, dimension(size(var,1)-1, size(var,2)) :: d_x

      im  = size(var,1)
      d_x = (var(2:im,:) - var(1:im-1,:))/dx 
    end function d_x
    
!--------------- derivative_y ------------------
    function d_y (var)
      real, dimension(:,:) :: var
      real, dimension(size(var,1), size(var,2)-1) :: d_y

      jm  = size(var,2)
      d_y = (var(:,2:jm) - var(:,1:jm-1))/dy 
    end function d_y
    
!------------------ expand_x ------------------------
    function e_x(var, fac)
      integer :: fac
      real, dimension(:,:) :: var
      real, dimension(size(var,1)+2, size(var,2)) :: e_x

      im = size(var,1)
      e_x(2:im+1, : ) =  var
      e_x(     1, : ) =  var( 1,:)*fac
      e_x(  im+2, : ) =  var(im,:)*fac
    end function e_x

!------------------ expand_y ------------------------
    function e_y(var, fac)
      integer :: fac
      real, dimension(:,:) :: var
      real, dimension(size(var,1), size(var,2)+2) :: e_y

      jm = size(var,2)
      e_y( :, 2:jm+1) =  var
      e_y( :,      1) =  var(:, 1)*fac
      e_y( :,   jm+2) =  var(:,jm)*fac
    end function e_y
end module
!-----------------------------------------------------
!   Linear Shallow water Eq.
!-----------------------------------------------------
module sw2_eq
   use sw2_data

      real    :: omega=0.5
      integer :: nwave = 3

   contains
!------------- Runge-Kutta ------------
    subroutine runge_kutta
             
      real, dimension(im1,jm1) :: dh1, dh2, dh3, dh4
      real, dimension(imx,jm1) :: du1, du2, du3, du4
      real, dimension(im1,jmx) :: dv1, dv2, dv3, dv4
     
      call drv(u       , v       , h       , du1, dv1, dh1)
      call drv(u+du1/2., v+dv1/2., h+dh1/2., du2, dv2, dh2)
      call drv(u+du2/2., v+dv2/2., h+dh2/2., du3, dv3, dh3)
      call drv(u+du3   , v+dv3   , h+dh3   , du4, dv4, dh4)
        
      u = u + (du1 + 2.*du2 + 2.*du3 + du4) / 6.
      v = v + (dv1 + 2.*dv2 + 2.*dv3 + dv4) / 6.
      h = h + (dh1 + 2.*dh2 + 2.*dh3 + dh4) / 6.
      p_time = p_time + dt

      end subroutine

!------------- Shallow Water Eq. ------------
    subroutine drv(u, v, h, du, dv, dh)
      real, dimension(imx,jm1) :: u, du
      real, dimension(im1,jmx) :: v, dv
      real, dimension(im1,jm1) :: h, dh

      if((f==0.).and.(b==0.))then
      du=dt*(-g*d_x(h))
      dv=dt*(-g*d_y(h))
      dh = dt * (-d*(d_x(e_x(u,-1)) + d_y(e_y(v,-1))) + kelvin())

      else if((f==0.).and.(b/=0.))then
      du=dt*(-g*d_x(h))
      dv=dt*(-g*d_y(h))
      dh = dt * (-d*(d_x(e_x(u,-1)) + d_y(e_y(v,-1))) + kelvin())

      else if((f/=0.).and.(b==0.))then
      du=dt*(-g*d_x(h))
      dv=dt*(-g*d_y(h))
      dh = dt * (-d*(d_x(e_x(u,-1)) + d_y(e_y(v,-1))) + kelvin())

      else
      du = dt * ( f*a_y(a_x(e_y(v,-1))) - g*d_x(h))
      dv = dt * (-f*a_x(a_y(e_x(u,-1))) - g*d_y(h))
      dh = dt * (-d*(d_x(e_x(u,-1)) + d_y(e_y(v,-1))) + kelvin())
    
    end if

    end subroutine

!-------------- Kelvin wave maker --------------
    function kelvin()
      real, parameter :: pi=3.14159
      real, dimension(im1,jm1) :: kelvin
      
      kelvin = 0.
      phase = p_time*omega
      if (phase > 2*pi*nwave) return
    
   if((f==0.).and.(b/=0.))then
   kelvin(1,:)=sin(phase)*exp((-yh**n)*abs(b))/dx
                                                           
   else if((f==0.).and.(b==0.))then
    kelvin(1,1)=3*sin(abs(phase))/dx
    kelvin(2,1)=kelvin(1,1)/2.
    kelvin(1,2)=kelvin(1,1)/2.
    kelvin(2,2)=kelvin(1,1)/(2.*sqrt(2.))

 
   else if((f/=0.).and.(b==0.))then
    kelvin(1,1)=3*sin(abs(phase))/dx
    kelvin(2,1)=kelvin(1,1)/2.
    kelvin(1,2)=kelvin(1,1)/2.
    kelvin(2,2)=kelvin(1,1)/(2.*sqrt(2.))

   else
   kelvin(1,:)=sin(phase)*exp(-yh/r0)/dx

   end if

    end function

end module

program main
  use sw2_eq
    call setup

!!!   write(*,*) 'omega '
!!!   read (*,*)  omega
  
!  ---- time integration ----

    call swlset('lalt' , .true.)
    call swlset('lwait', .false.)
    call gropn(1)

    do ii=1, 100
      do j=1, 5
        call runge_kutta
      end do
      call graph
    end do
    call grcls
    

contains

!------------------ graphic output ------------------
  subroutine graph
    character msg*12
    character msgf*12
 
    call grfrm
    call grswnd( 0., xmax, 0., ymax )
    call uspfit
    call grstrf
    call ussttl('X','','Y','')
    call usdaxs
    write(msg,'(a,f7.2)')'t=',p_time
    call uxmttl('t',msg,-1.)
    write(msgf,'(a,f7.2)')'f=',f
    call uxmttl('t',msgf,1.)

    call udgcla(-2., 2., 0.1)
    call udcntr( h, im1, im1, jm1) 
    
  end subroutine


end program










